% function SegmentPattern_MONKS(input_image, output_corr_file)
%
%  Finding correspondences between the input_image and the projected
%  pattern
%  Stores the file output_corr_file with a list of the correspondences
%  found.
%
% Example: 
% SegmentPattern_MONKS('images/monks_horse1_aligned_median3.jpg', 'correspondences/corr_monks_horse2.dat');
%
function SegmentPattern_MONKS(input_image, output_corr_file)

addpath('segmentation');

LeftMarginPattern = 64; %black margin width in pixels


%read input image:
img = imread(input_image);


%------- IVC tests --------
%img = imread('IVC/monks_plane1_aligned.jpg');

%-----------------------------------------------------------------
%THRESHOLD TO DECIDE IF A PIXEL BELONGS TO THE BACKGROUND OR NOT:
%one of the channels must be greater
MINIMA_COMPONENT = 70.0;
%-----------------------------------------------------------------

i=imshow(img);
p=get(i,'Parent');
fig_pattern=get(p, 'Parent');
axis on;
hold on;

rows = length(img(:,1,1));
cols = length(img(1,:,1));

number_maxima = [];
number_minima = [];
number_edges = [];

%we get two points defining the area of interest:
[COL, ROW] = ginput(2);
COL = round(COL);
ROW = round(ROW);

% COL = [200; 900];  %ICIP (calibration1) plane test
% ROW = [200; 600];

% COL = [230; 725];  %IVC (calibration1) plane test
% ROW = [390; 680]; 
% 

correspondences = [];

for row=ROW(1):5:ROW(2),  
   
    image_row = [];
    for c = 1:cols,
        image_row = [image_row; img(row, c, 1) img(row, c, 2) img(row, c, 3)];
    end
    
    [Peaks, PeakType, colorSegmented_row, matchings] = Segment_Pattern_Row(image_row, COL(1), COL(2), 0);

    if (length(matchings)>0)
       for i=1:length(matchings(:,1)),
           correspondences = [correspondences; matchings(i,1) row matchings(i,2)];
       end
        
        num_max = 0;
       
        %-------- draw the extrema found ---------
        figure(fig_pattern);
        for i=1:length(Peaks),
            if (Peaks(i)>COL(1)) & (Peaks(i)<COL(2))
                if (PeakType(i)==1) %maximum                
                    mx = max([image_row(round(Peaks(i)),1),image_row(round(Peaks(i)),2),image_row(round(Peaks(i)),3)]);
                    if (mx>MINIMA_COMPONENT)
                        plot(Peaks(i), row, '+r');
                    end
                    num_max = num_max + 1;
                end
            end
        end
        number_maxima = [number_maxima; num_max];   
    end
end
number_peaks = [number_maxima]'

%store correspondences 
correspondences(:,3) = correspondences(:,3) + LeftMarginPattern; %we add the left margin of the projected pattern

%filtrate the peaks belonging to the background:
bak = [];
for i=1:length(correspondences(:,1)),
    xc = round(correspondences(i,1));
    yc = round(correspondences(i,2));
    if (max([img(yc,xc,1), img(yc,xc,2), img(yc,xc,3)])>MINIMA_COMPONENT)
        bak = [bak; correspondences(i,:)];
        plot(correspondences(i,1), correspondences(i,2), 'ob');
    end
end

correspondences = bak;

fid = fopen(output_corr_file, 'w');
fprintf(fid, '%d\r\n', length(correspondences(:,1)));
fprintf(fid, '%f %f %f\r\n', correspondences');
fclose(fid);
