%
%function prn2byuDELAUNAY(nom_fitxer, output_file, threshold)
%
%   Generates a byu format file from a prn file using delaunay
%   triangulation. Only saves all those triangles whose sides are
%   shorter than 'threshold'
%
% nom_fitxer: 'fitxer.prn'
% output_file: name of the output file
%
% prn2byuDELAUNAY('cavall1_filtered.prn', 'cavall1_filtered.g', 40);
%
function prn2byuDELAUNAY(nom_fitxer, output_file, threshold)
%per treure axis desde la linia de comandes del matlab un cop apareixen les figures:
%axis off

fid = fopen(nom_fitxer,'r');
npunts = fscanf(fid,'%d',1);
P = fscanf(fid,'%f',[6,npunts]);
fclose(fid);

%si ho tenim en coordenades de cmara:
Xc = P(4,:); %eixos de la cmara
Yc = P(5,:);
Zc = P(6,:);	
X = Zc;  %eixos del matlab
Y = Xc;
Z = -Yc;	

Pixel = [X; Y];
C = Z;

clear tri;
tri=delaunay(Y, Z);
ntriangles = length(tri(:,1));


num_triangles_eliminats = 0;
triangle_eliminat = zeros(1,ntriangles);

for i=1:ntriangles,
    longitud_1 = sqrt((X(tri(i,1))-X(tri(i,2)))^2+(Y(tri(i,1))-Y(tri(i,2)))^2+(Z(tri(i,1))-Z(tri(i,2)))^2);
    longitud_2 = sqrt((X(tri(i,1))-X(tri(i,3)))^2+(Y(tri(i,1))-Y(tri(i,3)))^2+(Z(tri(i,1))-Z(tri(i,3)))^2);
    longitud_3 = sqrt((X(tri(i,2))-X(tri(i,3)))^2+(Y(tri(i,2))-Y(tri(i,3)))^2+(Z(tri(i,2))-Z(tri(i,3)))^2);
    
    if (longitud_1>threshold)|(longitud_2>threshold)|(longitud_3>threshold)
        num_triangles_eliminats=num_triangles_eliminats+1;
        triangle_eliminat(1,i) = 1;
    end
end



%tri=Triangles';

%-----------------------------
%    BYU format
%-----------------------------
output = fopen(output_file,'w+');
%save header:
fprintf(output, '1 %d %d %d\n', npunts, ntriangles-num_triangles_eliminats, 3*ntriangles);
fprintf(output, '1 %d\n', ntriangles);
%save points: x y z
for i=1:npunts,
    fprintf(output, '%12.5f %12.5f %12.5f\n', X(i), Y(i), Z(i));
end
%save triangles:
for i=1:ntriangles,
    if (triangle_eliminat(1,i)==0)
        fprintf(output, '%d %d %d\n', tri(i,1), tri(i,2), -tri(i,3));
    end
end
fclose(output);
