%function number = CountZeroCrossings(vector)
%
% returns the number of zero-crossings found in the given vector
% and their subpixel positions
function [number, positions] = CountZeroCrossings(vector)

number = 0;
positions = [];

for c=2:length(vector),
    if (vector(c)==0)  %exact zero-crossing (it is rare but it happened to me!!)
        number = number + 1;
        positions = [positions; c];
    end    
    if (vector(c)>0) & (vector(c-1)<0) %zero-crossing from negative to positive
        estimate = c; %the estimate is always at right of the subpixel zero-crossing
        %we fulfill linear interpolation between the two consecutive pixels
        %and calculate its zero-crossing
        delta = vector(estimate-1)/(vector(estimate-1)-vector(estimate)) - 1;        
        if (abs(delta)>2.0)
            delta = 0.0;
        end
        number = number + 1;
        positions = [positions; c+delta];
    end
    if (vector(c)<0) & (vector(c-1)>0) %zero-crossing from positive to negative
        estimate = c; %the estimate is always at right of the subpixel zero-crossing
        %we fulfill linear interpolation between the two consecutive pixels
        %and calculate its zero-crossing
        delta = vector(estimate-1)/(vector(estimate-1)-vector(estimate)) - 1;        
        if (abs(delta)>2.0)
            delta = 0.0;
        end        
        number = number + 1;
        positions = [positions; c+delta];
    end    
end
