function [Peaks, PeakType, colorSegmented_row, correspondences] = Segment_Pattern_Row(image_row_RGB, starting_column, ending_column, ShowPlots);

WidthStripe=7;

addpath('./segmentation/DynamicProgramming');

colorSegmented_row = [];
correspondences = [];

minimum_MAX_channel_intensity = 70.0; 

%------------------------------
%   CONFIGURATION PARAMETERS
%------------------------------
order =4;  %derivative filter

%--- threshold_second_derivative ----%
% it is used to differentiate regions in the M channel
% corresponding to maxima and minima
% if the second derivative is greater than the threshold is a maxima region
% if the second derivate is lower than the negative threshold is a minima
% region:
threshold_2nd_derivative = 0; %it is enough
saturation_interval_2nd_derivative = 0.2; %the second derivative will be saturated at the interval [-threshold, threshold]
minimum_pattern_intensity = 0.1;   %minimum intensity for considering a pixel belonging to a pattern

period_colors = zeros(4,3);
period_colors(1,:) = [255 0 0];
period_colors(2,:) = [128 255 0];
period_colors(3,:) = [0 255 255];
period_colors(4,:) = [255 0 255];
%-------------------------------------------------------------------------

cols = length(image_row_RGB);

%----------------------------------------------------
%  EXTRACT THE RGB CHANNELS FROM THE GIVEN IMAGE ROW
%----------------------------------------------------
R = double(image_row_RGB(:,1))/255; %we normalize the channels in the interval [0..1]
G = double(image_row_RGB(:,2))/255;
B = double(image_row_RGB(:,3))/255;

R_normalized = R;
G_normalized = G;
B_normalized = B;

d_R = DerivateVector(R, order)';
d_G = DerivateVector(G, order)';
d_B = DerivateVector(B, order)';

%-------------------------------------------------------------------------
% Calculation of the MAX channel defined as M = max(R,G,B) for each pixel
%-------------------------------------------------------------------------
M = [];
for i=1:cols,
    M = [M max([R(i) G(i) B(i)])];
end

%first derivative
d_M = DerivateVector(M, order);

%channel used to detect the edges:
%combined = sqrt(d_R.^2 + d_G.^2 + d_B.^2 + d_M.^2);
combined = d_M.^2;
d_combined = DerivateVector(combined, order);

%we search zero-crossings in this channel:
[num_zero_d_combined, pos_zero_d_combined] = CountZeroCrossings(d_combined);  %we search zero-crossing in the first derivative of the combined channel
num_zero_d_combined
%now we only store the descending zero-crossings (maxima peaks in combined channel)
maxima_channel_combined = [];
for m=1:num_zero_d_combined,
    if (d_combined(floor(pos_zero_d_combined(m)))>0) & (d_combined(ceil(pos_zero_d_combined(m)))<0)
        if (pos_zero_d_combined(m)>starting_column) & (pos_zero_d_combined(m)<ending_column)
            maxima_channel_combined = [maxima_channel_combined; pos_zero_d_combined(m)];
        end
    end
end

edges = maxima_channel_combined;

%----------------------------------------
%  MAXIMA SEARCH between certain edges
%----------------------------------------
maxima = [];
maxima_color = [];
for i=1:length(edges)-1,
    %we look at the first derivative of the MAX channel: if between two
    %edges most part of d_M is descending --> there is a maximum
    starting = ceil(edges(i));
    ending = floor(edges(i+1));
    
    if (starting>=starting_column) & (ending<=ending_column)
        
        segment_d_M = d_M(starting:ending);
        
        %[num_zero, pos_zero] = CountZeroCrossings(segment_d_M);
        %if (num_zero==1) & (segment_d_M(round(pos_zero)-1)>0) & (segment_d_M(round(pos_zero)+1)<0)
        
        num_descending = 0;
        num_ascending = 0;
        for c = 1:length(segment_d_M)-1,
            if (segment_d_M(c)>segment_d_M(c+1))
                num_descending = num_descending + 1;
            else
                num_ascending = num_ascending + 1;
            end
        end
        
        if (num_descending > num_ascending)
            M(starting:ending) = M(starting:ending)/max(M(starting:ending));
            segment = M(starting:ending);
            maximum =  Normalized_centroid(segment, 0.85);
            maxima = [maxima; maximum+starting-1];
            maximum_color = [R(round(maximum+starting-1)) G(round(maximum+starting-1))  B(round(maximum+starting-1))];
            %normalize color:
            maximum_color = maximum_color/max(maximum_color);
            maxima_color = [maxima_color;  maximum_color];
        end
    end
end

%-----------------------------------------------
% PLOTS
%-----------------------------------------------
if (ShowPlots==1)
  fig1=figure;
  axis([1 cols -2 2]);
  hold on;    

  plot(1:cols, M, 'magenta');
  plot(1:cols, combined, 'red');
  
  for i=1:cols,
      if (d_M(i)>1)
          d_M(i) = 1;
      end
      if (d_M(i)<-1)
          d_M(i) = -1;
      end
  end
  
  plot(1:cols, d_M, 'blue');
  
  for i=1:length(edges),
      plot(edges(i), 0, 'g+');
  end
  for i=1:length(maxima),
      plot(maxima(i), 0, 'k+');
  end  
end


maxima_color;
for i=1:length(maxima_color),
    [m, pos_min] = min(maxima_color(i,:));
    maxima_color(i,pos_min) = 0;
end

%-----------------------------------------
%   STORE THE RESULTS
%-----------------------------------------
Peaks = maxima;   %first the maxima
PeakType = [];
for i=1:length(maxima),
    PeakType = [PeakType; 1];  %1 identifies a maximum
end


%---------------------------------------------------------
%   SOLVING CORRESPONDENCES THROUGH DYNAMIC PROGRAMMING
%---------------------------------------------------------

% %calculate image scanline in terms of the alphabet codification (each type
% %of stripe color is encoded with a letter of the alphabet:
% image_scanline = '';
% for i=1:length(maxima_color(:,1)),
%     symbol = alphabet_symbol(maxima_color(i,1), maxima_color(i,2), maxima_color(i,3));
%     image_scanline = strcat(image_scanline, symbol);
% end
% image_scanline
% 

load('stripe_RGB.mat', 'stripe_RGB');

emitted = stripe_RGB;
received = maxima_color;

%---------------------------------
%we try to identify the colors (if there had no cross-talk it would be no
%necessary since dynamic programming should work properly!, but we have to
%help it a bit)
for i=1:length(received(:,1)),
    [newR, newG, newB, j] = IdentifyRGB(received(i,:),[1 0 0; 0.5 1.0 0; 0.0 1.0 1.0; 0.5 0.0 1.0]);
    received(i,:) = [newR newG newB];
end
%---------------------------------

%the components of emitted and received are R G B; R G B.... of every
%projected and perceived stripe. Each RGB component is expressed in [0..1]
%we now rescale this interval to [-1..1]:
for i=1:3,
    emitted(:,i) = emitted(:,i)*2-1;
    received(:,i) = received(:,i)*2-1;    
end


[C, M] = CalculateCost_and_PathMatrices(emitted,received);

%M2 = [round(image_scanline)' M];
%paths = [M2; 0 round(pattern_scanline(35:35+12))]
%paths = [M2; 0 round(pattern_scanline)];


matchings = TrackOptimalPath(C,M); %we have matchings of type: (index edge projected, index edge received)
%disp('matchings: emitted index  -   received index   -    RGB emitted   -  RGB received');
matching_results = [];
for i=1:length(matchings(:,1)),
    matching_results = [matching_results; matchings(i,:) emitted(matchings(i,1),:)   received(matchings(i,2),:)];
end
matching_results;
    
% if (length(matchings(:,1))==29)
%     matchings(:,1) = (46:-1:18)';
% end
% if (length(matchings(:,1))==30)
%     matchings(:,1) = (46:-1:17)';
% end
% 
% matchings
matchings

correspondences = [];
%we must transform the matchings to pixel coordinates:
for i=length(matchings(:,1)):-1:1,    
    correspondences = [correspondences; Peaks(matchings(i,2))    (2*matchings(i,1)-1)*WidthStripe+WidthStripe/2.0];
end

